********************************************************************
* lwasm -r --pragma=condundefzero -o/tmp/dw.bin dw.asm -b -l > /tmp/dw.lst
*
* 2014 Tormod Volden
*
* DriveWire "dw" command dweeb for dwload
* Usage example: DLOAD"DW""DW SERVER STATUS"

Start    equ   $0E00

* DriveWire protocol constants
OP_SERSETSTAT equ $C4
OP_SERWRITE equ $C3
OP_SERREADM equ $63
OP_SERREAD equ $43
SS.Open  equ   $29
SS.Close equ   $2A
 
* Entry point
         org   Start

Entry
* Get bitbanger routines from calling DWLOAD
	 ldu   ,s		get stacked PC
	 ldx   1,u
         stx   DWReadv
	 ldx   3,u
         stx   DWWritev
	 ldx   5,u
	 stx   DoReadv

* parse command arguments
         jsr   <$a5             peek at next char
         lbeq   noargs

         * jsr   $8E51            Get8Bit into B

	 jsr   GETFNAM
         ldx   #pretxt-1
	 jsr   OUTSTR
	 ldu   #namebuf+1
         ldb   -1,u		* read length
         lda   #13		* append CR
	 sta   b,u
         incb
         clr   b,u		* zero termination byte

	lda   #1		* Virtual channel 1
* open channel
	ldb   #SS.Open
	pshs  a,b
	lda   #OP_SERSETSTAT
	pshs  a
	leax  ,s
	ldy   #3
	jsr   [DWWritev]
* send string
	lda   #OP_SERWRITE
	sta   ,s
@nxt	lda   ,u+
	beq   strend
	sta   2,s
	leax  ,s
	ldy   #3
	jsr   [DWWritev]
	bra   @nxt
strend

* leave some time for server to write to channel
	ldx   $A000
@delay	leax  -1,x
	bne   @delay
	
* get reply bytes until empty channel
readrep	lda   #OP_SERREAD
	sta   ,s
	leax  ,s
	ldy   #1
	jsr   [DWWritev]
* get response bytes
getres  leax  ,s
	ldy   #2
	jsr   [DWReadv]
	lbcs   Err
	lbne   Err
        lda   ,x
	lbeq   ret	* no data or status for any channel
	cmpa  #16
	blo   singl
	beq   vsstat
	cmpa  #32
	blo   multb
	cmpa  #128
	lbeq   ret	* no data or status for any channel
* VWINDOW
	bra   Err	* not implemented
singl	deca		* ignore channel number for now
	lda   1,x	* just print the byte
	jsr   OUTCHR
	bra   readrep
multb	suba  #17	* channel
	ldb   1,x	* number of bytes
	* use OP_SERREADM ...
	sta   1,s	* channel
	stb   2,s	* count
	lda   #OP_SERREADM
	sta   ,s
	leax  ,s
	ldy   #3
	clra
	pshs  a,b	* count in 16 bit
	jsr   [DWWritev]
	ldy   ,s
	ldx   #namebuf	* recycle buffer for input
	jsr   [DWReadv]
	bcs   Err
	bne   Err
	puls  d		* count in b
*	clr   d,x
*	leax  -1,x
*	jsr   OUTSTR
@nxc	lda   ,x+
	jsr   OUTCHR
	decb
	bne   @nxc
	bra   readrep   * keep polling
	
vsstat	lda   1,x	* status byte
	cmpa  #15
	bhi   Err	* reboot request?
	beq   allch
*	cmpa  ?,s	* compare with our channel number
*	bne   Err
	bra   chclos

read1b
	lda   #1
	sta   1,s	* channel
	sta   2,s	* count
	lda   #OP_SERREADM
	sta   ,s
	leax  ,s
	ldy   #3
	jsr   [DWWritev]
	leax  ,s
	ldy   #1
	jsr   [DWReadv]
	bcs   Err
	bne   Err
        lda   ,x
	jsr   OUTCHR
	lbra   readrep   * keep polling

allch	ldx   #allcltxt-1
	bra   pri
chclos
	adda  #'0
	jsr   OUTCHR
	ldx   #clostxt-1
	bra   pri
Err	ldx   #errtxt-1
pri	jsr   OUTSTR
ret	leas  3,s
	rts

noargs   ldx   #nonetxt-1
         jsr   OUTSTR
         rts

pretxt   fcn   /DW /
nonetxt  fcn   /NO TEXT GIVEN/
errtxt   fcn   /ERR/
allcltxt fcs   /ALL/
clostxt  fcn   / CLOSED/

* ROM routines used
OUTCH    equ   $BCAB    print char A to screen
OUTCHR   equ   $B54A    print char A to DEVNUM
OUTSTR   equ   $90E5    print string at X+1 to DEVNUM
OUTNUM   equ   $957A    print number in D to DEVNUM
GETFNAM  equ   $B7AA    read file name and length into $1d1 (max 31 chars)
IOERROR  equ   $B84B    IO_error
SYSERR   equ   $8344    system error

namebuf  equ   $1d1 fixed by BASIC read filename function which stores in $1d1

DWReadv  fdb   0
DWWritev fdb   0
DoReadv  fdb   0

         end   Entry
